<?php
	
	global $genders;
	if (!isset($genders))
		$genders = array ();
	$genders['male'] = array (
		'e'      => 'he',
		'em'     => 'him',
		'eir'    => 'his',
		'eirs'   => 'his',
		'emself' => 'himself',
	);
	$genders['female'] = array (
		'e'      => 'she',
		'em'     => 'her',
		'eir'    => 'her',
		'eirs'   => 'hers',
		'emself' => 'herself',
	);
	foreach ($genders as &$g)
		foreach ($g as $k => $v)
			$g[ucfirst($k)] = ucfirst($v);
	
	class Yandere_theme
	{
		public $name;
		
		public $game_name;
		public $game_name_full;
		public $game_name_specific;
		public $game_name_plural;
		
		protected $role_singular = array ();
		protected $role_plurals = array ();
		protected $standard_roles = array ();
		
		protected $role_upgrades = array ();
		protected $role_reveal_as = array ();
		protected $role_actions = array (
			0 => array (),
			1 => array (),
		);
		protected $action_messages = array (
			0 => array (),
			1 => array (),
		);
		
		protected $enemy_roles = array ();
		protected $always_win_roles = array ();
		protected $guard_friendly = array ();
		
		protected $yandere_role_odds = array ();
		protected $role_odds = array ();
		protected $single_use_roles = array ();
		protected $role_appearances = array ();
		protected $role_descriptions = array ();
		
		protected $weapons = array ();
		protected $murder_descriptions = array ();
		protected $generic_victims = array ();
		
		protected $command_remaps = array (
			'kill' => 'kill|vote',
		);
		
		// Feature support flags; read-only
		protected $role_appearances_multi = true;
		protected $role_chooser_version = 2;
		
		//== Constructor ==//
		
		public function __construct ()
		{
		}
		
		public function all_roles ()
		{
			return array_keys($this->role_descriptions);
		}
		
		public function role_singular ($role)
		{
			$single = $this->role_singular[$role];
			if (isset($single))
				return $single;
			if (preg_match('/^[aeiou]/', $role))
				return "an $role";
			return "a $role";
		}
		
		public function role_fromsingular ($role)
		{
			$e = explode(' ', $role, 2);
			return $e[1];
		}
		
		public function role_plural ($role)
		{
			$plural = $this->role_plurals[$role];
			if (isset($plural))
				return $plural;
			return $role . 's';
		}
		
		public function role_fromplural ($role)
		{
			return array_search($role, $this->role_plurals);
		}
		
		public function get_standard_role ($stdrole)
		{
			return $this->standard_roles[$stdrole];
		}
		
		public function role_upgrade ($role)
		{
			$newrole = $this->role_upgrades[$role];
			if (!isset($newrole))
				return $newrole;
			
			if (!is_array($newrole))
				return $newrole;
			
			$odds = array();
			foreach ($newrole as $nro)
			{
				$odd = $this->role_odds[$nro];
				if (!isset($odd))
					$odd = $this->yandere_role_odds[$nro];
				if (!isset($odd))
					$odd = 1;
				$odds[$nro] = $odd;
			}
			
			return array_rand_weighted($odds);
		}
		
		public function upgrade_reveal_role ($role)
		{
			$re = $this->role_reveal_as[$role];
			if (isset($re))
				return $re;
			return $role;
		}
		
		public function role_may ($role, $phase, $action, $target, $actedtimes)
		{
			$OK = $this->role_actions[$phase][$role];
			if (is_array($OK))
			{
				if (!$action)
					$action = $OK['default'];
				if (!$action)
					// OBSOLETE
					$action = $this->role_default_actions[$role];
				if (!$action)
					$action = $OK['default'];
				if (!$action)
					$action = $OK[0];
				return in_array($action, $OK) ? $action : false;
			}
			if ($OK == $action || !$action)
				return $OK;
			return false;
		}
		
		protected function victim_role ($target, $blind)
		{
			if ($blind)
				return ','.($target->is_mafioso ? '' : ' who is not').' '.$this->role_singular($this->get_standard_role('mafioso')).',';
			return ', a '.$target->role.',';
		}
		
		public function action_message ($role, $phase, $action, $who, $target, $blind)
		{
			$AM = $this->action_messages[$phase][$role];
			if (is_array($AM))
				$AM = $AM[$action];
			
			$victim_role = null;
			if (strpos($AM, '\\3'))
				$victim_role = $this->victim_role($target, $blind);
			
			$AM = strtr($AM, array ('\\1' => $who->nick, '\\2' => $target->nick, '\\3' => $victim_role));
			
			return $AM;
		}
		
		public function role_is_enemy ($role)
		{
			return in_array($role, $this->enemy_roles);
		}
		
		public function role_is_mafioso ($role)
		{
			return array_key_exists($role, $this->yandere_role_odds);
		}
		
		// role_supports_team returns >=:
		//      0 -- the role OPPOSES the team
		//     20 -- the role supports the team winning IN GENERAL
		//     60 -- the role supports ONLY this team winning
		public function role_supports_team ($role, $team)
		{
			if ($this->role_always_wins($role))
				return 20;
			$is_enemy = $this->role_is_enemy($role);
			switch ($team) {
			case 0:  // Honest
				return $is_enemy ? 0 : 90;
			case 1:  // Mafioso/Yandere
				return $is_enemy ? 90 : 0;
			}
			return 0;
		}
		
		// role_sustains_team returns >=:
		//      0 -- the team lives without this role
		//     40 -- the role sustains the team's continued existence
		public function role_sustains_team ($role, $team)
		{
			return ($this->role_is_mafioso($role) == ($team == 1)) ? 40 : 0;
		}
		
		public function role_is_deadly_to_guard ($role)
		{
			return $this->role_is_mafioso($role) && !in_array($role, $this->guard_friendly);
		}
		
		public function role_is_single_use ($role)
		{
			return in_array($role, $this->single_use_roles);
		}
		
		public function role_always_wins ($role)
		{
			return in_array($role, $this->always_win_roles);
		}
		
		public function prepare_chooser(&$priv, $playercount, $team, $teamcount)
		{
			switch ($this->role_chooser_version) {
			case 0:
			case 1:
				$priv = array(
					'version' => 1,
					'detectives' => floor($playercount / 14) + 1,
					'specials' => 1,
				);
				break;
			default:
			$this->my_role_chooser_init($priv, $playercount, $team, $teamcount);
			}
		}
		
		public function choose_mafioso_role (&$priv)
		{
			if (isset($priv) && $priv['team'] == 1)
				return $this->my_role_chooser($priv);
			
			return array_rand_weighted($this->yandere_role_odds);
		}
		
		public function choose_role (&$priv)
		{
			if (isset($priv))
			{
				if ($priv['team'] === 0)
				return $this->my_role_chooser($priv);
				else
				if ($priv['version'] == 1)
				{
					if ($priv['detectives'])
					{
						--$priv['detectives'];
						return $this->get_standard_role('detective');
					}
					else
					if ($priv['specials'])
					{
						--$priv['specials'];
						return $this->choose_special_role($priv);
					}
				}
			}
			
			return array_rand_weighted($this->role_odds);
		}
		
		public function choose_special_role (&$priv)
		{
			if (isset($priv) && $priv['team'] === 0)
				// FIXME: Deprecate this function and/or fix this to exclude 'innocent' role
				return $this->my_role_chooser($priv);
			
			$role_odds = $this->role_odds;
			$innocent = $this->get_standard_role('innocent');
			unset ($role_odds[$innocent]);
			return array_rand_weighted($role_odds);
		}
		
		public function choose_role_appearance ($role)
		{
			$appearance = $this->role_appearance($role);
			if (is_array($appearance))
				return $appearance[array_rand($appearance)];
			return $appearance;
		}
		
		public function role_appearance ($role)
		{
			$appearance = $this->role_appearances[$role];
			if (isset($appearance))
				return $appearance;
			return $role;
		}
		
		public function role_description ($role, $channel)
		{
			if (!array_key_exists($role, $this->role_descriptions))
				return false;
			$desc = $this->role_descriptions[$role];
			$CS = call('yandere/get-channel-strings', $channel, array('Bot', '!'));
			$desc = strtr($desc, array (
				'\\1' => $CS['Bot'],
				'\\2' => call('yandere/get-format-command', $channel, null),
				'\\3' => call('yandere/get-format-command', null, null),
			));
			return $desc;
		}
		
		public function describe_death ($victim, $blind)
		{
			$desc = ''
			. $this->murder_descriptions[mt_rand(0, count($this->murder_descriptions) - 1)]
			. ' '
			. $this->weapons[mt_rand(0, count($this->weapons) - 1)]
			;
			
			if ($victim)
				$desc = $victim->nick . $this->victim_role($victim, $blind) . " $desc";
			
			return $desc;
		}
		
		public function generic_victim ()
		{
			$victim = $this->generic_victims[array_rand($this->generic_victims)];
			$gender = $victim['gender'];
			if (isset($gender))
			{
				global $genders;
				foreach ($genders[$gender] as $k => $v)
				{
					if (!isset($victim[$k]))
						$victim[$k] = $v;
				}
			}
			return $victim;
		}
		
		public function get_introduction ($phase)
		{
			$GV = $this->generic_victim();
			return $GV['name'] . ' ' . $this->describe_death(null, false) . '! We need to find the '.$this->get_standard_role('mafioso').'.';
		}
		
		public function remap_command($role, $phase, $priv, $command, $vote, $user)
		{
			return $this->command_remaps[$command];
		}
		
		// NOTE: really private, but made public FOR TESTING ONLY
		public function calculate_role_value($role, $playercount)
		{
			$appearance = $this->role_appearance($role);
			$is_enemy = $this->role_is_enemy($role);
			
			$value = 0;
			
			if (!is_array($appearance))
				$appearance = array($appearance);
			$AAEcount = 0;
			foreach ($appearance as $appear)
				if ($this->role_is_enemy($appear))
					++$AAEcount;
			$appear_as_enemy = $AAEcount / count($appearance);
			
			if ($is_enemy)
				$value += 1.25 * (1 - $appear_as_enemy);
			else
				$value -= $appear_as_enemy;
			
			$actPhases = 0;
			$yanderecount = floor(($playercount - 4) / 3) + 1;
			foreach ($this->role_actions as $phase => $pra)
			{
				if (!isset($pra[$role]))
					continue;
				$pas = $pra[$role];
				if (!is_array($pas))
					$pas = array($pas);
				if ($phase != 1 || $pas != array('vote'))
					++$actPhases;
				foreach ($pas as $pa)
				{
					$value += .2;
					switch ($pa) {
					case 'kill':
						if ($phase == 0)
							$value += .5;
						break;
					case 'check':
						$value += .1;
						break;
					case 'spy':
						$value += .2;
						break;
					case 'stalk':
						$value += .3;
						break;
					case 'upgrade':
						$value += .4;
						break;
					case 'vote':
						$value += 1 / $yanderecount;
						// FIXME: This needs to propagate to multiuse too somehow
						break;
					}
				}
			}
			if (in_array($role, $this->single_use_roles))
				$value += .8;
			// TODO: multiple/counted use roles just get the .8 multiplied
			else
			if ($actPhases)
				// Scale regular usage
				$value += ceil(($playercount - 2) / 2) * .8 * $actPhases;
			
			$revealAs = $this->upgrade_reveal_role($role);
			if ($this->role_upgrade($role) === null && in_array($revealAs, $this->role_reveal_as))
			{
				if ($is_enemy)
					$value += .1;
				else
				$value -= .1;
			}
			
			return $value;
		}
		
		// NOTE: really private, but made public FOR TESTING ONLY
		public function calculate_role_odds($role_values, $points, $remaining, $best_role_value)
		{
			$role_odds = array();
			if ($points < 0)
				$target_value = $points;
			else
				$target_value = $points * 2 / 3;
			$failover = array();
			$maxdist = abs($target_value - $best_role_value);
			$maxdist = max($maxdist, abs($target_value));
			foreach ($role_values as $role => $value)
			{
				$role_odds[$role] = pow(($maxdist + 1) - abs($value - $target_value), 5);
// echo "$role_odds[$role] Odds $role $value (goal $target_value)\n";
			}
			
			return $role_odds;
		}
		
		// NOTE: really private, but made public FOR TESTING ONLY
		public function my_role_chooser_init(&$chooser, $playercount, $team, $teamcount)
		{
			$roles = $this->all_roles();
			$role_values = array();
			$sust_role_values = array();
			$supp_role_values = array();
			$best_role_value = -99;
			foreach ($roles as $role)
			{
				$support = $this->role_supports_team($role, $team);
				if ($support < 20)
					continue;
				$value = $this->calculate_role_value($role, $playercount);
				
				if ($support < 60)
					// Penalty on baka rangers
					$value -= 2;
				
				$role_values[$role] = $value;
				if ($value > $best_role_value)
					$best_role_value = $value;
				
				$sustain = $this->role_sustains_team($role, $team);
				if ($sustain < 40)
					continue;
				$sust_role_values[$role] = $value;
				if ($support < 60)
					continue;
				$supp_role_values[$role] = $value;
			}
			
			$best_role_chance = .65;
			if ($team)
				$best_role_chance = .3;
			$points = $best_role_value * (3 / 2) * $best_role_chance;
			
			// FIXME: this should REALLY be a class/object :/
			$chooser = array(
				'teamcount'        => $teamcount       ,
				     'role_values' => $role_values     ,
				'sust_role_values' => $sust_role_values,
				'supp_role_values' => $supp_role_values,
				'best_role_value'  => $best_role_value ,
				'points'           => $points          ,
				'chosen'           => array()          ,
				'team'             => $team            ,
			);
		}
		
		// NOTE: really private, but made public FOR TESTING ONLY
		public function my_role_chooser(&$chooser)
		{
			$teamcount        = $chooser['teamcount'       ];
			     $role_values = $chooser[     'role_values'];
			$sust_role_values = $chooser['sust_role_values'];
			$supp_role_values = $chooser['supp_role_values'];
			$best_role_value  = $chooser['best_role_value' ];
			$points           = $chooser['points'          ];
			$i = $teamcount - count($chooser['chosen']);
			
				if ($i > ($teamcount / 2) - 1)
					$active_role_values = $supp_role_values;
				else
				if ($i > $teamcount / 4)
					$active_role_values = $sust_role_values;
				else
					$active_role_values = $role_values;
				$role_odds = $this->calculate_role_odds($active_role_values, $points, $i, $best_role_value);
				$role = array_rand_weighted($role_odds);
echo "ROLE: $role; $points ($i) $best_role_value\n";
			$chooser['chosen'][] = $role;
				$points -= $role_values[$role];
			
			$chooser['points'] = $points;
			return $role;
		}
		
		// NOTE: really private, but made public FOR TESTING ONLY
		public function inside_choose_roles($playercount, $team, $teamcount)
		{
			$chooser = null;
			$this->my_role_chooser_init($chooser, $playercount, $team, $teamcount);
			
			$teamroles = array();
			for ($i = $teamcount; $i; --$i)
				$teamroles[] = $this->my_role_chooser($chooser);
			
			return $teamroles;
		}
		
		private function rename_role_A (&$RK, &$old, &$new)
		{
			$b = array_search($old, $RK);
			if ($b !== false)
				$RK[$b] = $new;
			return $b;
		}
		
		private function rename_role_B (&$RK, &$old, &$new)
		{
			foreach ($RK as $phase => &$L)
				$this->rename_role_C($L, $old, $new);
		}
		
		private function rename_role_C (&$L, &$old, &$new)
		{
			if (!array_key_exists($old, $L))
				return;
			$L[$new] = $L[$old];
			unset($L[$old]);
		}
		
		private function rename_role_D (&$RK, &$old, &$new)
		{
			foreach ($RK as &$RKi)
			{
				if (is_array($RKi))
					$this->rename_role_A($RKi, $old, $new);
				else
				if ($RKi == $old)
					$RKi = $new;
			}
		}
		
		// Usage: rename_role('detective', 'spy');
		// Usage: rename_role(array('detective' => 'spy'));
		protected function rename_role ($a, $new=null, $desc=false, $plurals=null)
		{
			if (isset($new))
				$a = array($a => $new);
			
			if (!isset($plurals))
				$plurals = array ();
			
			foreach ($a as $old => $new)
			{
				if (array_key_exists($old, $this->role_plurals))
					unset($this->role_plurals[$old]);
				if (array_key_exists($new, $plurals))
					$this->role_plurals[$new] = $plurals[$new];
				$this->rename_role_A($this->standard_roles   , $old, $new);
				$this->rename_role_B($this->role_actions     , $old, $new);
				$this->rename_role_B($this->action_messages  , $old, $new);
				
				$this->rename_role_A($this->enemy_roles      , $old, $new);
				$this->rename_role_A($this->always_win_roles , $old, $new);
				$this->rename_role_A($this->guard_friendly   , $old, $new);
				
				$this->rename_role_C($this->yandere_role_odds, $old, $new);
				$this->rename_role_C($this->role_odds        , $old, $new);
				$this->rename_role_A($this->single_use_roles , $old, $new);
				$this->rename_role_C($this->role_upgrades    , $old, $new);
				$this->rename_role_D($this->role_upgrades    , $old, $new);
				$this->rename_role_C($this->role_appearances , $old, $new);
				while (false !== $this->rename_role_A($this->role_appearances, $old, $new))
					;
				
				if ($desc === true)
				{
					$this->rename_role_C($this->role_descriptions, $old, $new);
					$this->role_descriptions[$new] =
						str_replace($old, $new,
						str_replace(ucfirst($old), ucfirst($new),
						str_replace(strtoupper($old), strtoupper($new),
						$this->role_descriptions[$old]
					)));
				}
				else
				if (is_array($desc))
					$this->role_descriptions[$new] = $desc[$new];
				else
				if ($desc !== false)
					$this->role_descriptions[$new] = $desc;
				unset($this->role_descriptions[$old]);
			}
		}
	}
	
?>
