<?php
	
	list ($command, $user_id, $vote) = $irc_call_args;
	
	if ($command !== false && is_null($command))
	{
		$command = strtolower($command);
	}
	
	$channel = irc_lower($to);
	
	if (!isset ($this->custom_vars->yandere_games[$channel]))
	{
		output($to, $from.': There\'s no game here.');
		return;
	}
	
	$game = $this->custom_vars->yandere_games[$channel];
	
	if (!isset ($game->players[$user_id]))
	{
		output($to, $from.': You aren\'t in this game.');
		return;
	}
	
	if (!$game->active)
	{
		output($to, $from.': The game hasn\'t started yet.');
		return;
	}
	
	if ($game->turn % 2 == 1)
	{
		// Trying to write a command to the channel during the night. Not allowed.
		
		$CS = call('yandere/get-channel-strings', $to, array('the channel', 'PM(v)'));
		
		output($to, $game->s.'Commands in '.$CS['the channel'].' are ignored during the night. Please '.$CS['PM(v)'].' them to me instead. This message does not imply that '.$game->players[$user_id]->nick.' has any special ability.'.$game->e);
		return;
	}
	
	if ($vote === false || is_null($vote))
	{
		$vote_id = $vote;
	}
	else
	{
		switch ($command)
		{
		case 'vote':
		case 'kill|vote':
			break;
		
		default:
			notice($to, $game->s.'That action isn\'t allowed during the day.'.$game->e);
			return;
		}
		
		$vote_id = call('yandere/get-user-id', $vote);
		
		if (!isset ($game->players[$vote_id]))
		{
			output($to, $from.': '.$vote.' isn\'t in this game.');
			return;
		}
	}
	
	if ($vote_id == $user_id)
	{
		output($to, $game->s.'You can\'t vote for yourself.'.$game->e);
		return;
	}
	
	if (is_null($vote_id))
	{
		if (!isset ($game->votes[$user_id]))
		{
			notice($from, $game->s.'You haven\'t voted yet.'.$game->e);
			return;
		}
	}
	elseif (isset ($game->votes[$user_id]) && $game->votes[$user_id] == $vote_id)
	{
		notice($from, $game->s.'You already voted that way. The vote will be tallied when the day ends.'.$game->e);
		return;
	}
	
	if (isset ($game->votes[$user_id]))
	{
		$old_vote_name = ($game->votes[$user_id] !== false) ? $game->players[$game->votes[$user_id]]->nick : 'abstain';
	}
	else
	{
		$old_vote_name = null;
	}
	
	if ($vote_id === false)
	{
		$vote_name = 'abstain';
	}
	elseif (is_null($vote_id))
	{
		$vote_name = 'undecided';
	}
	else
	{
		$vote_name = $game->players[$vote_id]->nick;
	}
	
	if (is_null($vote_id))
	{
		unset ($game->votes[$user_id]);
	}
	else
	{
		$game->votes[$user_id] = $vote_id;
	}
	
	// Tally the current vote count, including the new one being cast.
	
	$abstains = 0;
	$current_votes = array ();
	
	foreach ($game->votes as $value)
	{
		if ($value === false)
		{
			$abstains++;
		}
		elseif (!isset ($current_votes[$value]))
		{
			$current_votes[$value] = 1;
		}
		else
		{
			$current_votes[$value]++;
		}
	}
	
	foreach ($current_votes as $key => $value)
	{
		$current_votes[$key] = $game->players[$key]->nick.': '.$value;
	}
	
	$current_votes[] = 'Abstains: '.$abstains;
	$current_votes[] = 'Undecided: '.(count($game->players) - count($game->votes));
	
	// Output the message.
	
	if (is_null($vote_id))
	{
		output($to, $game->s.$from.' has switched from '.$old_vote_name.' to undecided. '.implode(', ', $current_votes).$game->e);
	}
	elseif (!is_null($old_vote_name))
	{
		output($to, $game->s.$from.' has switched from '.$old_vote_name.' to vote '.($vote_id === false ? 'to' : 'for').' '.$vote_name.'. '.implode(', ', $current_votes).$game->e);
	}
	else
	{
		output($to, $game->s.$from.' has voted '.($vote_id === false ? 'to' : 'for').' '.$vote_name.'. '.implode(', ', $current_votes).$game->e);
	}
	
	// Determine whether or not the turn is over.
	
	if (count($game->votes) != count($game->players))
	{
		// There is a player who hasn't cast a vote yet.
		
		return;
	}
	
	$vote_pick = call('yandere/solve-day-vote', $channel);
	
	if (is_null($vote_pick))
	{
		// Everyone voted, but there was no resolution.
		
		output($to, $game->s.'Everyone has voted, but there is a tie. Someone will need to switch their vote, or you can type "'.call('yandere/get-format-command', $to, 'next').'" and force the group to abstain.'.$game->e);
	}
	else
	{
		// All players have voted and there is a resolution. Begin the night (which will kill the voted player).
		
		call('yandere/night', $channel, $vote_pick);
	}
	
?>
