<?php
	
	list ($command, $user_id, $vote) = $irc_call_args;
	
	if ($command !== false && !is_null($vote))
	{
		$command = strtolower($command);
	}
	
	// Let's see if the user is in a game somewhere.
	
	$channel = call('yandere/get-player-channel', $user_id);
	
	if (is_null($channel))
	{
		output($to, $from.': You aren\'t in any '.$this->custom_vars->yandere_global_theme->game_name_plural.'.');
		return;
	}
	
	$game = $this->custom_vars->yandere_games[$channel];
	
	if (!$game->active)
	{
		output($to, $from.': The game hasn\'t started yet.');
		return;
	}
	
	$phase = $game->turn % 2;
	
	$user = $game->players[$user_id];
	
	if (is_null($user->get_can_act($phase, false, null, true)))
	{
		notice($to, $game->s.'You don\'t have a '.($phase == 1 ? 'night' : 'day').' role.'.$game->e);
		return;
	}
	
	$role = $user->role;
	
	if ($vote === false || is_null($vote))
	{
		$vote_id = $vote;
	}
	else
	{
		switch ($command)
		{
		case 'hide':
		case 'reveal':
			$vote_id = $user_id;
			break;
		
		default:
			$vote_id = call('yandere/get-user-id', $vote);
			
			if (!isset ($game->players[$vote_id]))
			{
				output($to, $from.': '.$vote.' isn\'t in this game.');
				return;
			}
			
			if ($vote_id == $user_id)
			{
				notice($to, $game->s.'You can\'t do that to yourself.'.$game->e);
				return;
			}
		}
	}
	
	$command = explode('|', $command);
	foreach ($command as $potcmd)
	{
		if ($user->get_can_act($phase, $potcmd, null))
		{
			$command = $potcmd;
			break;
		}
	}
	if (is_array($command))
		$command = $command[0];
	
	if ($vote !== false && !is_null($vote))
	{
		if (false === $user->get_can_act($phase, $command, $target))
		{
			if ($command == 'vote')
				notice($to, $game->s.'That command isn\'t allowed in PM.'.$game->e);
			else
			if ($user->did_action)
			{
				notice($to, $game->s.'You can\'t do that. You can only use this role\'s ability once per ' . ($user->single_use ? 'game' : ($phase ? 'night' : 'day')) . '.'.$game->e);
			}
			else
			{
				notice($to, $game->s.'You can\'t do that.'.$game->e);
			}
			return;
		}
	}
	
	if ($game->forbid_early_kill && $command == 'kill' && $game->turn < 2 && !$user->is_enemy)
	{
		notice($to, $game->s.'You can\'t use this channel until the first night has passed.'.$game->e);
		return;
	}
	
	if ($vote_id !== false && !is_null($vote_id))
	{
		$target = $game->players[$vote_id];
	}
	
	if ($phase == 0)
	{
		// Day.
		
		if ($vote_id === false || is_null($vote_id))
		{
			return;
		}
		
		switch ($command)
		{
		case 'check':
			notice($user->nick, $game->s.$target->nick.' '.($target->is_enemy ? 'seems' : 'does not seem').' to be evil.'.$game->e);
			break;
		
		case 'kill':
			$msg = $game->theme->action_message($user->role, $phase, $command, $user, $target, $game->blind_roles);
			
			output($channel, $game->s.$msg.$game->e);
			
			if (isset ($game->votes[$vote_id]))
			{
				unset ($game->votes[$vote_id]);
			}
			
			foreach ($game->votes as $key => $value)
			{
				if ($value == $vote_id)
				{
					unset ($game->votes[$key]);
				}
			}
			
			if ($game->restrict_voice)
			{
				$this->send('MODE '.$channel.' -v '.$game->players[$vote_id]->nick);
			}
			
			unset ($game->players[$vote_id]);
			break;
		
		case 'reveal':
			if ($user->has_upgrade_reveal)
				$msg = $user->nick . " lights up bright and is revealed to be an upgraded " . $user->upgrade_reveal_role . "!";
			else
			$msg = $game->theme->action_message($user->role, $phase, $command, $user, $target, false);
			
			output($channel, $game->s.$msg.$game->e);
			break;
		
		case 'spy':
			notice($user->nick, $game->s.$target->nick.' looks like a '.$target->appearance.'.'.$game->e);
			break;
		
		case 'upgrade':
			if ($target->has_upgrade_reveal)
			{
				notice($to, $game->s.$target->nick.' has already been upgraded as much as possible.'.$game->e);
				return;
			}
			
			$newrole = $game->theme->role_upgrade($target->role);
			if (isset($newrole))
			{
				$target->role = $newrole;
				$msg = 'You have been upgraded to ' . $game->theme->role_singular($newrole) . '. ' . call('yandere/describe-user', $game->theme, $target, $channel);
			}
			else
			{
				$target->has_upgrade_reveal = true;
				$msg = 'You have been upgraded! You may now publicly reveal yourself to be ' . $game->theme->role_singular($target->upgrade_reveal_role) . (($target->upgrade_reveal_role == $target->role) ? '' : ' (even though you\'re still really ' . $game->theme->role_singular($target->role) . ')') . ' by typing "' . call('yandere/get-format-command', null, 'reveal') . '" during the day.';
			}
			$target->did_action = false;
			notice($to, $game->s.$target->nick.' has been upgraded.'.$game->e);
			notice($target->nick, $game->s.$msg.$game->e);
			break;
		
		default:
			notice($to, $game->s."Error: $command command not implemented at day.".$game->e);
			return;
		}
		
		if (!($command == 'reveal' && $user->has_upgrade_reveal))
		{
			$user->did_action = true;
		}
		
		if (call('yandere/check-game-over', $channel, true))
		{
			return;
		}
	}
	else
	{
		// Night.
		
		if (is_null($vote_id))
		{
			if (!isset ($game->votes[$user_id]))
			{
				notice($from, $game->s.'You haven\'t decided anything yet.'.$game->e);
				return;
			}
		}
		elseif (isset ($game->votes[$user_id]) && $game->votes[$user_id] == $vote_id)
		{
			// FIXME: this should check the already-done action
			if ($user->get_can_act($phase, 'vote', $target))
			{
				notice($from, $game->s.'You already voted that way. The vote will be tallied when the '.($game->turn % 2 == 0 ? 'day' : 'night').' ends.'.$game->e);
			}
			else
			{
				notice($from, $game->s.'You already said that. The action will be carried out when the night ends.'.$game->e);
			}
			return;
		}
		
		if (!isset ($game->votes[$user_id]))
		{
			$old_vote_name = null;
		}
		elseif ($game->votes[$user_id] === false)
		{
			$old_vote_name = 'abstain';
		}
		elseif ($command == 'hide')
		{
			$old_vote_name = 'hiding';
		}
		else
		{
			$old_vote_name = $game->players[$game->votes[$user_id]]->nick;
		}
		
		if (is_null($vote_id))
		{
			$vote_name = 'undecided';
		}
		elseif ($vote_id === false)
		{
			$vote_name = 'abstain';
		}
		elseif ($command == 'hide')
		{
			$vote_name = 'hiding';
		}
		else
		{
			$vote_name = $target->nick;
		}
		
		if (is_null($vote_id))
		{
			unset ($game->votes[$user_id]);
		}
		else
		{
			$game->votes[$user_id] = $vote_id;
		}
		
		$voters = array ();
		
		foreach ($game->players as $player_id => $player)
		{
			if ($player->get_can_act($phase, 'vote', null))
			{
				$voters[$player_id] = $player;
			}
		}
		
		if ($user->get_can_act($phase, 'vote', null) && count($voters) > 1)
		{
			foreach ($voters as $player_id => $player)
			{
				$who = $player_id == $user_id ? 'You\'ve' : $user->nick.' has';
				
				if (is_null($vote_id))
				{
					output($player->nick, $game->s.$who.' switched from '.$old_vote_name.' to undecided. '.implode(', ', $current_votes).$game->e);
				}
				elseif (!is_null($old_vote_name))
				{
					notice($player->nick, $game->s.$who.' switched from '.$old_vote_name.' to '.($vote_id === false ? '' : 'target ').$vote_name.'.'.$game->e);
				}
				elseif ($vote_id === false)
				{
					notice($player->nick, $game->s.$who.' decided to abstain.'.$game->e);
				}
				else
				{
					notice($player->nick, $game->s.$who.' cast a vote for '.$vote_name.'.'.$game->e);
				}
			}
		}
		else
		{
			if (is_null($vote_id))
			{
				output($to, $game->s.'You\'ve switched from '.$old_vote_name.' to undecided.'.$game->e);
			}
			elseif (!is_null($old_vote_name))
			{
				notice($from, $game->s.'You\'ve switched from '.$old_vote_name.' to '.(($vote_id !== false && !is_null($vote_id) && $command != 'hide') ? 'target ' : '').$vote_name.'.'.$game->e);
			}
			elseif ($vote_id === false)
			{
				notice($from, $game->s.'You\'ve decided to abstain.'.$game->e);
			}
			elseif ($command == 'hide')
			{
				notice($from, $game->s.'You\'ve decided to hide.'.$game->e);
			}
			else
			{
				notice($from, $game->s.'You\'ve set your target to '.$vote_name.'.'.$game->e);
			}
		}
		
		foreach ($game->players as $player_id => $player)
		{
			if ($player->get_can_act($phase, false, null) && !isset ($game->votes[$player_id]))
			{
				// There is a night role player who hasn't played yet.
				
				return;
			}
		}
		
		// All night roles have played. Begin the day.
		
		call('yandere/day', $channel);
	}
	
?>
