<?php
	
	list ($params) = $irc_call_args;
	
	if (!isset ($this->custom_vars->yandere_games))
	{
		$this->custom_vars->yandere_games = array ();
		$this->custom_vars->yandere_themes = array ();
		$this->custom_vars->yandere_default_theme = require_once(dirname(__FILE__)."/../themes/default.php");
		require_once(dirname(__FILE__)."/../themes/" . $this->custom_vars->yandere_default_theme . ".php");
		$YTC = 'Yandere_theme_' . $this->custom_vars->yandere_default_theme;
		$this->custom_vars->yandere_global_theme = new $YTC;
	}
	
	clearstatcache();
	
	$user_id = call('yandere/get-user-id', $from);
	
	if (is_null($user_id))
	{
		output($to, $from.': For some reason, I don\'t know your hostmask. This is UNPOSSIBLE and means I have a bug. And you can\'t play until it\'s fixed. :(');
		return;
	}
	
	$privmsg = $to == $from;
	$channel = irc_lower($to);
	if ($privmsg)
		$channel = call('yandere/get-player-channel', $channel);
	$game = $this->custom_vars->yandere_games[$channel];
	$theme = isset($game) ? $game->theme : $this->custom_vars->yandere_global_theme;
	
	if (count($params) && isset($game))
	{
		$user = $game->players[$user_id];
		$role = $user->role;
		$phase = $game->turn % 2;
		$privmsg = $to == $from;
		$command = $params[0];
		$vote = $params[1];
		
		$remap = $game->theme->remap_command($role, $phase, $privmsg, $command, $vote, $user);
		
		if (is_array($remap))
			$params = $remap;
		else
		if (isset($remap))
			$params[0] = $remap;
	}
	
	switch (count($params))
	{
	case 0:
		call('yandere/join', $user_id, array ());
		return;
	
	case 1:
		switch (strtolower($params[0]))
		{
		case 'abstain':
			call('yandere/action', 'abstain', $user_id, false);
			return;
		
		case 'end':
		case 'exit':
		case 'quit':
		case 'stop':
			$channel = irc_lower($to);
			
			if (!isset ($this->custom_vars->yandere_games[$channel]))
			{
				output($to, $from.': There\'s no game here.');
				return;
			}
			
			$game = $this->custom_vars->yandere_games[$channel];
			
			if ($game->starter != $user_id && !$this->is_admin($from))
			{
				output($to, $from.': You don\'t have permission to do that.');
				return;
			}
			
			output($to, $game->s.ucfirst($game->theme->game_name_specific).' game has been forced to end.'.$game->e);
			
			call('yandere/end-game', $channel);
			
			return;
		
		case 'extend':
			$channel = irc_lower($to);
			
			if (!isset ($this->custom_vars->yandere_games[$channel]))
			{
				output($to, $from.': There\'s no game here.');
				return;
			}
			
			$game = $this->custom_vars->yandere_games[$channel];
			
			$game->max_extensions++;
			
			output($to, 'Added another waiting extension.');
			return;
		
		case 'faster':
		case 'hurry':
		case 'next':
			$channel = irc_lower($to);
			
			if (!isset ($this->custom_vars->yandere_games[$channel]))
			{
				output($to, 'There\'s no game here.');
				return;
			}
			
			$game = $this->custom_vars->yandere_games[$channel];
			
			if (!isset ($game->players[$user_id]))
			{
				// User isn't in the game.
				
				notice($from, 'You aren\'t in this game.');
				return;
			}
			
			if (!$game->active)
			{
				if (count($game->players) < 4)
				{
					output($to, 'There aren\'t enough players yet.');
					return;
				}
				
				call('yandere/start-game', $channel, $game->start_time, 0);
				return;
			}
			
			if ($game->turn % 2 == 0)
			{
				if (count($game->votes) == count($game->players))
				{
					// Everyone has voted. Move on immediately.
					
					call('yandere/night', $channel, call('yandere/solve-day-vote', $channel));
				}
				elseif (!$game->hurried)
				{
					// Not everyone has voted. Wait a bit.
					
					if (!in_array($user_id, $game->hurry_votes))
					{
						$game->hurry_votes[] = $user_id;
						
						if (count($game->hurry_votes) / count($game->players) >= .5)
						{
							$game->hurried = true;
							
							$players_pending = call('yandere/players-pending', $game);
							
							output($channel, $game->s.'People are getting impatient! You all have 1 minute to cast your vote before the day ends. We\'re still waiting on '.implode_comma_series($players_pending).'.'.$game->e);
							
							$this->schedule_event(time() + 60, array ('yandere/day-timeout', $channel, $game->start_time, $game->turn));
						}
						else
						{
							$remaining = ceil(count($game->players) * .5) - count($game->hurry_votes);
							
							notice($from, 'You have voted to hurry the day. '.$remaining.' more ' . str_plural('vote is', $remaining, 'votes are') . ' required.');
						}
					}
					else
					{
						$remaining = ceil(count($game->players) * .5) - count($game->hurry_votes);
						
						notice($from, 'You already voted to hurry the day. '.$remaining.' more ' . str_plural('vote is', $remaining, 'votes are') . ' required.');
					}
					
				}
			}
			elseif (!$game->hurried)
			{
				$game->hurried = true;
				
				output($channel, $game->s.'People are getting impatient! The night roles have 1 minute to make their decisions before the night ends.'.$game->e);
				
				call('yandere/night-reminders', $game);
				
				$this->schedule_event(time() + 60, array ('yandere/night-timeout', $channel, $game->start_time, $game->turn));
			}
			
			return;
		
		case 'help':
			$channel = irc_lower($to);
			
			$CS = call('yandere/get-channel-strings', $to, array('PM conversations', 'PM messages', 'the channel'));
			$CS['yandere'] = $theme->get_standard_role('mafioso');
			$CS['yanderes'] = $theme->role_plural($CS['yandere']);
			
			$gameplay = 'This is a detective game. It cycles between two phases: During the day, everyone gets together and tries to figure out who they think is a '.$CS['yandere'].', and they can vote to lynch someone. During the night, players with special roles can perform actions that may shed some light.';
			
			$rules = 'Each night, the '.$CS['yanderes'].' can pick one player to kill. The object of the game is to find and kill the '.$CS['yanderes'].' before they kill you. '.ucfirst($CS['PM conversations']).' between players is allowed at any time, but you MAY NOT copy/paste '.$CS['PM messages'].' into '.$CS['the channel'].'.';
			
			if (!isset ($this->custom_vars->yandere_games[$channel]))
			{
				output($to, ucfirst($theme->game_name).' is a variation of the classic Mafia game, also known as Werewolf. Type "'.call('yandere/get-format-command', $to, array(array('', 'join', 'start'))).'" to start a game.');
				
				output($to, $gameplay);
				output($to, $rules);
				return;
			}
			
			$game = $this->custom_vars->yandere_games[$channel];
			
			if (!$game->active)
			{
				output($to, ucfirst($game->theme->game_name) . ' is a variation of the classic Mafia game, also known as Werewolf. A new game is about to start. Type "'.call('yandere/get-format-command', $to, array(array('', 'join', 'start'))).'" to join.');
				
				output($to, $gameplay);
				output($to, $rules);
			}
			elseif (!isset ($game->players[$user_id]))
			{
				notice($from, 'Please wait until the game is over and then type "'.call('yandere/get-format-command', $to, array(array('', 'join', 'start'))).'" to play.');
				
				notice($from, $gameplay);
				notice($from, $rules);
			}
			else
			{
				$player = $game->players[$user_id];
				$role = $player->role;
				
				notice($from, 'You\'re '.$game->theme->role_singular($role).'. '.call('yandere/describe-user', $game->theme, $player, $to));
				
				notice($from, $gameplay);
				notice($from, $rules);
			}
			return;
		
		case 'hide':
			call('yandere/action', 'hide', $user_id, $user_id);
			return;
		
		case 'reveal':
			call('yandere/action', 'reveal', $user_id, $user_id);
			return;
		
		case 'undecided':
		case 'unvote':
			call('yandere/action', 'vote', $user_id, null);
			return;
		}
		break;
	
	case 2:
		$command = strtolower($params[0]);
		
		switch ($command)
		{
		case 'alignment':
		case 'check':
			call('yandere/action', 'check', $user_id, $params[1]);
			return;
		
		case 'assassinate':
		case 'attack':
		case 'battle':
		case 'destroy':
		case 'fight':
		case 'kill':
		case 'kill|vote':
		case 'murder':
		case 'rape':
		case 'slaughter':
			call('yandere/action', 'kill|vote', $user_id, $params[1]);
			return;
		
		case 'defend':
		case 'guard':
		case 'protect':
			call('yandere/action', 'guard', $user_id, $params[1]);
			return;
		
		case 'esp':
		case 'examine':
		case 'google':
		case 'infiltrate':
		case 'look':
		case 'snoop':
		case 'spy':
			call('yandere/action', 'spy|check', $user_id, $params[1]);
			return;
		
		case 'upgrade':
			call('yandere/action', 'upgrade', $user_id, $params[1]);
			return;
		
		case 'follow':
		case 'stalk':
		case 'watch':
			call('yandere/action', 'stalk', $user_id, $params[1]);
			return;
		
		case 'vote':
			call('yandere/action', 'vote', $user_id, $params[1]);
			return;
		}
		break;
	}
	
	switch (strtolower($params[0]))
	{
	case 'begin':
	case 'join':
	case 'start':
		call('yandere/join', $user_id, array_slice($params, 1));
		return;
	
	case 'help':
		call('yandere/help', implode(' ', array_slice($params, 1)));
		return;
	
	case 'force':
		if (!$this->is_admin($from))
		{
			output($to, $from.': You don\'t have permission to do that.');
			return;
		}
		if ($from == $to)
			$to = $params[1];
		$from = $params[1];
		call('yandere/main', array_slice($params, 2));
		return;
	
	default:
		if (count($params) == 1)
		{
			if ($to == $from)
			{
				$channel = call('yandere/get-player-channel', $user_id);
				
				if (is_null($channel))
				{
					// User isn't playing a game on this network.
					
					break;
				}
			}
			else
			{
				$channel = irc_lower($to);
				
				if (!isset ($this->custom_vars->yandere_games[$channel]))
				{
					// There is no game in this channel.
					
					break;
				}
			}
			
			$game = $this->custom_vars->yandere_games[$channel];
			
			if (!isset ($game->players[$user_id]))
			{
				// The user isn't in this game.
				
				break;
			}
			
			if ($to != $from || is_null($action = $game->players[$user_id]->get_can_act(null, false, null/*FIXME TODO*/)))
			{
				$action = 'vote';
			}
			
			call('yandere/action', $action, $user_id, $params[0]);
			return;
		}
	}
	
	if ($to == $from)
	{
		notice($to, $from.': I don\'t recognize that command.');
	}
	else
	{
		output($to, $from.': I don\'t recognize that command.');
	}
	
?>
