<?php
	
	list ($channel, $start_time, $extensions) = $irc_call_args;
	
	if (!isset ($this->custom_vars->yandere_games[$channel]))
	{
		return;
	}
	
	$game = $this->custom_vars->yandere_games[$channel];
	
	if ($start_time != $game->start_time)
	{
		// This message is no longer relevant.
		return;
	}
	
	if ($game->active)
	{
		// The game already started.
		return;
	}
	
	if (count($game->players) < 4)
	{
		if ($extensions < $game->max_extensions)
		{
			output($channel, $game->s.'We don\'t have enough players yet. We need at least 4 players to start a game. I\'ll wait for another 60 seconds. Please type "'.call('yandere/get-format-command', $channel, array(array('', 'join', 'start'))).'" to join.'.$game->e);
			
			$this->schedule_event(time() + 60, array ('yandere/start-game', $channel, $game->start_time, $extensions + 1));
		}
		else
		{
			output($channel, $game->s.'There aren\'t enough players to start ' . $game->theme->game_name_full . '. Try again later.'.$game->e);
			
			unset ($this->custom_vars->yandere_games[$channel]);
		}
		
		return;
	}
	
	if (is_null($game->start_mode))
	{
		$game->start_mode = count($game->players) % 2 == 1 ? 'night' : 'day';
	}
	
	if ($game->restrict_voice)
	{
		// Silence all users in the channel.
		
		$this->send('MODE '.$channel.' +m');
	}
	
	call('yandere/remove-voice', $channel);
	
	$game->active = true;
	
	// Get a random-order list of the players, to simplify random role assignment.
	
	$user_ids = array_keys($game->players);
	shuffle($user_ids);
	
	// Find out how many of each role we're going to have.
	
	$count = count($user_ids);
	
	$units = array ();
	
	$num_yanderes = floor(($count - 4) / 3) + 1;
	
	$rcpriv = null;
	
	$do_cprep = method_exists($game->theme, 'prepare_chooser');
	
	switch ($game->complexity)
	{
	case 'basic':
		$units[$game->theme->get_standard_role('mafioso')] = $num_yanderes;
		break;
	
	case 'normal':
		if ($do_cprep)
			$game->theme->prepare_chooser($rcpriv, $count, 1, $num_yanderes);
		for ($i = 0; $i < $num_yanderes; ++$i)
		{
			$yandere_type = $game->theme->choose_mafioso_role($rcpriv);
			
			++$units[$yandere_type];
		}
		break;
	}
	
	if (!$do_cprep || $game->complexity == 'basic')
		// NOTE: Assume if prepare_chooser exists, any such logic is in the chooser
	$units[$game->theme->get_standard_role('detective')] = floor($count / 14) + 1;
	
	$remaining = $count - array_sum($units);
	
	$innocent = $game->theme->get_standard_role('innocent');
	
	switch ($game->complexity)
	{
	case 'basic':
		$units[$innocent] = $remaining;
		break;
	
	case 'normal':
		if ($do_cprep)
			$game->theme->prepare_chooser($rcpriv, $count, 0, $remaining);
		
		$role = $game->theme->choose_special_role($rcpriv);
		++$units[$role];
		
		while (--$remaining)
		{
				$role = $game->theme->choose_role($rcpriv);
			
			++$units[$role];
		}
		break;
	}
	
	// Assign the roles.
	
	$unit_list = array ();
	
	$yandere_nicks = array ();
	$yandere_users = array ();
	
	$i = 0;
	
	foreach ($units as $role => $num)
	{
		if (!$game->blind_roles)
		{
			$unit_list[] = $num.' '.str_plural($role, $num, $game->theme->role_plural($role));
		}
		
		while ($num--)
		{
			$game->players[$user_ids[$i]]->role = $role;
			
			if ($game->theme->role_is_mafioso($role))
			{
				$user = $game->players[$user_ids[$i]];
				$yandere_users[] = $user;
				$yandere_nicks[] = $user->nick;
			}
			
			++$i;
		}
	}
	
	if ($game->blind_roles)
	{
		$role = $game->theme->get_standard_role('mafioso');
		
		$unit_list = array ($num_yanderes.' '.str_plural($role, $num_yanderes, $game->theme->role_plural($role)));
	}
	
	$game->init_players = $game->players;
	
	foreach ($game->players as $player)
	{
		notice($player->nick, $game->s.'You\'re '.$game->theme->role_singular($player->role).'. '.call('yandere/describe-user', $game->theme, $player, $to));
		
		if ($game->theme->role_is_mafioso($player->role))
		{
			notice($player->nick, $game->s.'The ' . $game->theme->role_plural($game->theme->get_standard_role('mafioso')) . ' in this game are: '.implode_comma_series($yandere_nicks).'.'.$game->e);
		}
	}
	
	// Introduction message
	
	output($channel, $game->s . $game->theme->get_introduction($game->turn) . $game->e);
	
	rsort($unit_list);
	
	$mafname = $game->theme->get_standard_role('mafioso');
	if ($units['yanderes'] != 1)
		$mafname = $game->theme->role_plural($mafname);
	
	output($channel, $game->s.'This game has '.implode_comma_series($unit_list).'. It\'s your job to determine the '.$mafname.'.'.$game->e);
	
	call('yandere/'.$game->start_mode, $channel);
	
?>
